const alertsService = Components.classes["@mozilla.org/alerts-service;1"].getService(Components.interfaces.nsIAlertsService);
const logInProgressLabels = ["loginDs", "loginNs", "authenticating",
                             "sendingTicket", "retrievingContactList"];

var Controller = {
  bundle : Utils.getStringBundle("chrome://msnmsgr/locale/msnmsgr.properties")
}

Controller.onContactAlreadySignedIn = function(aEmail) {
  window.moveContactToOnlineList(aEmail);
}

Controller.onContactAdded = function(aEmail, aDisplayName, aBlocked) {
  var label = aDisplayName;
  if (aBlocked) {
    var statusText = Utils.getStatusText("BLK");
    label += " (" + statusText + ")";
  }
  var statusCode = (aBlocked) ? "BLK" : "FLN";
  window.addContactToOfflineList(aEmail, label, statusCode);
}

Controller.onContactChanged
 = function(aEmail, aDisplayName, aStatus, aBlocked) {
  var statusCode = (aBlocked) ? "BLK" : aStatus;
  var label = aDisplayName;
  var statusText = Utils.getStatusText(statusCode);
  if (statusCode != "NLN" && statusCode != "FLN") {
    label += " (" + statusText + ")";
  }
  window.updateContact(aEmail, label, statusCode);
}

Controller.onContactListReceived = function() {
  window.showContactList();
}

Controller.onContactRemoved = function(aEmail) {
  window.removeContact(aEmail);
}

Controller.onContactSignedIn = function(aEmail, aDisplayName) {
  window.moveContactToOnlineList(aEmail);
  var shownDisplayName = "";
  if (aDisplayName.length > 20) {
    shownDisplayName = aDisplayName.substr(0, 20) + "...";
  }
  else {
    shownDisplayName = aDisplayName;
  }
  var icon = "chrome://msnmsgr/skin/msnmsgr_small.png";
  var title = this.bundle.GetStringFromName("msnNotification");
  var message = this.bundle.GetStringFromName("contactSignedIn")
                           .replace("%S", shownDisplayName);
  alertsService.showAlertNotification(icon, title, message, false, "", null);
}

Controller.onContactSignedOut = function(aEmail, aDisplayName, aBlocked) {
  window.moveContactToOfflineList(aEmail);
  this.onContactChanged(aEmail, aDisplayName, "FLN", aBlocked);
}

Controller.onDeleteContactRequest = function(aEmail, aDisplayName) {
  var topic = this.bundle.GetStringFromName("deleteContactQuestion");
  var question = this.bundle.GetStringFromName("deleteContactQuestion")
                            .replace("%S", aDisplayName).replace("%T", aEmail);
  if (Utils.askUser(topic, question)) {
    this.msnSession.removeContactFromForwardList(aEmail);
  }
}

Controller.onDisplayNameChanged = function(aDisplayName) {
  window.updateDisplayName(aDisplayName);
}

Controller.onErrorReceived = function(aErrorCode) {
  var title = window.bundle.GetStringFromName("error");
  var label = "error" + aErrorCode;
  var errorMessage = this.bundle.GetStringFromName(label);
  promptService.alert(window, title, errorMessage);
}

Controller.onLogInFailed = function(aErrorCode) {
  var title = window.bundle.GetStringFromName("error");
  var label = "error" + aErrorCode;
  var errorMessage = window.bundle.GetStringFromName(label);
  promptService.alert(window, title, errorMessage);
  window.signOut();
}

Controller.onLogInProgress = function(aStep) {
  var progressPercentage = aStep * 20;
  var progressLabel = logInProgressLabels[aStep];
  var progressText = this.bundle.GetStringFromName(progressLabel);
  window.updateProgressInfo(progressPercentage, progressText);
}

Controller.onLogInSuccessful = function() {
  var progressPercentage = 80;
  var progressLabel = logInProgressLabels[4];
  var progressText = this.bundle.GetStringFromName(progressLabel);
  window.updateProgressInfo(progressPercentage, progressText);
}

Controller.onLogOut = function() {
  delete this.msnSession;
  PrefObserver.unregister();
  window.reset();
}

Controller.onMessageStyleChanged = function() {
  for (var i in this.sbSessions) {
    this.sbSessions[i].onMessageStyleChanged();
  }
}

Controller.onNewEmailMessageCountChanged = function(nUnreadEmails) {
  var newEmailCountText = "";
  switch (nUnreadEmails) {
    case 0:
      newEmailCountText = this.bundle.GetStringFromName("noNewEmails");
      break;
    case 1:
      newEmailCountText = this.bundle.GetStringFromName("oneNewEmail");
      break;
    default:
      newEmailCountText = this.bundle.GetStringFromName("newEmails")
                                     .replace("%S", nUnreadEmails);
      break;
  }
  window.updateNewEmailMessageCount(newEmailCountText);
}

Controller.onNewEmailMessageReceived = function(aSender) {
  var icon = "chrome://msnmsgr/skin/msnmsgr_small.png";
  var title = this.bundle.GetStringFromName("mailNotification");
  var message = this.bundle.GetStringFromName("newEmailFrom")
                           .replace("%S", aSender);
  alertsService.showAlertNotification(icon, title, message, false, "", null);
}

Controller.onStatusChanged = function(aStatusCode) {
  var statusText = Utils.getStatusText(aStatusCode);
  var statusTextMessage = this.bundle.GetStringFromName("yourStatusMsg")
                                     .replace("%S", statusText);
  window.updateStatus(aStatusCode, statusTextMessage);
}

Controller.addNewContact = function() {
  var url = "chrome://msnmsgr/contact/addcontact.xul";
  Utils.openSingletonWindow(url, "msnsmgr:add-contact", "chrome",
                            this.msnSession);
}

Controller.blockContact = function(aEmail) {
  this.msnSession.blockContact(aEmail);
}

Controller.changeStatus = function(aStatus) {
  this.msnSession.changeStatus(aStatus);
}

Controller.logIn = function(aPassport, aPassword, aStatus) {
  var localeId = this.bundle.GetStringFromName("localeId");
  this.msnSession = new MsnSession(aPassport, aPassword, aStatus, localeId,
                                   this, MsnProtocol);
  MessageStyle.controller = this;
  MessageStyle.onStyleChanged();
  PrefObserver.register();
  this.msnSession.logIn();
}

Controller.logOut = function() {
  if (this.msnSession) {
    this.msnSession.logOut();
  }
}

Controller.openEmailInbox = function() {
  this.msnSession.openEmailInbox();
}

Controller.openEmailMessageComposeForm = function(aEmail) {
  this.msnSession.openEmailMessageComposeForm(aEmail);
}

Controller.openListsWindow = function() {
  var url = "chrome://msnmsgr/content/lists.xul";
  var features = "chrome, titlebar, centerscreen, resizable=yes, dialog=no";
  Utils.openSingletonWindow(url, "msnmsgr:lists", features, this.msnSession);
}

Controller.openPrefsWindow = function() {
  var url = "chrome://msnmsgr/content/prefs.xul";
  var features = "chrome, titlebar, centerscreen, resizable=yes";
  Utils.openSingletonWindow(url, "msnmsgr:prefs", features, this.msnSession);
}

Controller.openSbSession = function(aEmail) {
  this.msnSession.openSbSession(aEmail, null);
}

Controller.openWebMail = function(aAction, aFields) {
  var win = window.open("about:blank");
  var form = win.document.createElement("form");
  form.setAttribute("action", aAction);
  form.setAttribute("method", "POST");
  for (var field in aFields) {
    var input = win.document.createElement("input");
    input.setAttribute("type", "hidden");
    input.setAttribute("name", field);
    input.setAttribute("value", aFields[field]);
    form.appendChild(input);
  }
  win.document.body.appendChild(form);
  form.submit();
}