const FLAG_ACKNOWLEDGEMENT = 2;
const FLAG_SAVE_TO_FILE = 16;
const FLAG_NOTIFY_UI = 32;
const MSN_OBJECT_GUID = "{A4268EEC-FEC5-49E5-95C3-F126696BDBF6}";
const DISPLAY_PICTURE_APP_ID_1 = 1;
const DISPLAY_PICTURE_APP_ID_2 = 12;
const FILE_TRANSFER_GUID = "{5D3E02AB-6190-11D3-BBBB-00C04F795683}";
const FILE_TRANSFER_APP_ID = 2;
const BINARY_FOOTER_0 = "\0\0\0\0";

function P2P(aPassport, aSbSession) {
  this.passport = aPassport;
  this.sbSession = aSbSession;
  this.baseId = Math.round(Math.random() * 1000000);
  this.slpMsgBuffer = "";
  this.sessions = {};
  this.toAck = {};
}

P2P.prototype.onDisplayPictureReceived = function(aDisplayPicturePath) {
  this.sbSession.setDisplayPicture(aDisplayPicturePath);
}

P2P.prototype.onFileTransferred = function(aFileName) {
  this.sbSession.showFileTransferResult(0, aFileName);
}

P2P.prototype.onMessageReceived = function(aContactId, aMsg) {
  var binaryHeaders = aMsg.substr(0, 48);
  var headers = this.parseBinaryHeaders(binaryHeaders);
  var msg = aMsg.substr(48, headers.dataSize);
  if (headers.sessionId == 0) {
    if (headers.flags == FLAG_ACKNOWLEDGEMENT) {
      if (headers.dWord1 in this.toAck) {
        var sessionId = this.toAck[headers.dWord1];
        var session = this.sessions[sessionId];
        session.onAcknowledgementReceived();
      }
    }
    else {
      if (headers.dataSize == headers.msgLength) {
        this.sendAcknowledgement(aContactId, binaryHeaders);
        this.onSlpMessageReceived(aContactId, msg);
      }
      else {
        dump("Message divided in parts!\n");
        this.slpMsgBuffer += msg;
        if ((headers.dataOffset + headers.dataSize) == headers.msgLength) {
          this.sendAcknowledgement(aContactId, binaryHeaders);
          this.onSlpMessageReceived(aContactId, this.slpMsgBuffer);
          this.slpMsgBuffer = "";
        }
      }
    } 
  }
  else {
    if ((headers.dataOffset + headers.dataSize) == headers.msgLength
        && headers.flags != FLAG_ACKNOWLEDGEMENT) {
      this.sendAcknowledgement(aContactId, binaryHeaders);
    }
    if (headers.sessionId in this.sessions) {
      var session = this.sessions[headers.sessionId];
      session.onSessionMessageReceived(headers, msg);
    }
  }
}

P2P.prototype.onSlpMessageReceived = function(aContactId, aMsg) {
  dump("Received SLP message:\n" + aMsg + "-END-\n");
  var msg = this.parseSlpMessage(aMsg);
  var sessionId = msg.content["sessionid"];
  var session = this.sessions[sessionId];
  if (!session) {
    this.sessions[sessionId] = new P2PSession(sessionId, aContactId, this);
    session = this.sessions[sessionId];
  }
  if (msg.startLine == "MSNSLP/1.0 200 OK") {
    session.on200MessageReceived(msg.headers, msg.content);
  }
  else if (msg.startLine == "INVITE MSNMSGR:" + this.passport + " MSNSLP/1.0") {
    session.onInviteRequestReceived(msg.headers, msg.content);
  }
  else if (msg.startLine == "BYE MSNMSGR:" + this.passport + "MSNSLP/1.0") {
    this.onSessionClosed(sessionId);
  }
}

P2P.prototype.onSessionClosed = function(aSessionId) {
  delete this.sessions[aSessionId];
}

P2P.prototype.acceptFile = function(aFileName, aFileSize) {
  var file = this.sbSession.acceptFile(aFileName, aFileSize);
  return file;
}
P2P.prototype.getBinaryHeaders = function(aSessionId, aId, aDataOffset, aMsgLength, aDataSize, aFlags) {
  var binaryHeaders = convertIntTo4LeBytes(aSessionId)
    + convertIntTo4LeBytes(aId)
    + convertIntTo4LeBytes(aDataOffset) + "\0\0\0\0"
    + convertIntTo4LeBytes(aMsgLength) + "\0\0\0\0"
    + convertIntTo4LeBytes(aDataSize)
    + convertIntTo4LeBytes(aFlags)
    + convertIntTo4LeBytes(Math.round(Math.random() * 1000000))
    + "\0\0\0\0"
    + "\0\0\0\0\0\0\0\0";
  return binaryHeaders;
}

P2P.prototype.getEncodedContext = function(aFileName, aFileSize) {
  var headerLength = convertIntTo4LeBytes(574);
  var version = convertIntTo4LeBytes(2);
  var fileSize = convertIntTo4LeBytes(aFileSize) + "\0\0\0\0";
  var type = convertIntTo4LeBytes(1);
  var fileNameBytes = Utils.convertUTF16ToUTF16Bytes(aFileName);
  var nullsNeeded = 520 - fileNameBytes.length;
  for (var i = 0; i < nullsNeeded; i++) {
    fileNameBytes += "\0";
  }
  var unknownBase64Data = "\0\0\0\0\0\0\0\0"
   + "\0\0\0\0\0\0\0\0"
   + "\0\0\0\0\0\0\0\0"
   + "\0\0\0\0\0\0";
  var unknownDWord = 0xffffffff;
  var context = headerLength
    + version
    + fileSize
    + type
    + fileNameBytes
    + unknownBase64Data
    + unknownDWord;
  return context;
}

P2P.prototype.getDecodedContext = function(aContext) {
  var context = {};
  context.headerLength = convert4LeBytesToInt(aContext.substr(0, 4));
  context.version = convert4LeBytesToInt(aContext.substr(4, 4));
  context.fileSize = convert4LeBytesToInt(aContext.substr(8, 4))
  context.type = convert4LeBytesToInt(aContext.substr(16, 4));
  var fileNameBytes = aContext.substr(20, 520);
  var fileNameWithNulls = Utils.convertUTF16BytesToUTF16(fileNameBytes);
  var fileNameEndPos = fileNameWithNulls.indexOf("\0");
  context.fileName = fileNameWithNulls.substr(0, fileNameEndPos);
  return context;
}

P2P.prototype.getSlpMessage = function(aStartLine, aContactId, aBranchUid, aCSeq, aCallId, aContentType, aContent) {
  var slpMsg = aStartLine + "\r\n"
    + "To: <msnmsgr:" + aContactId + ">\r\n"
    + "From: <msnmsgr:" + this.passport + ">\r\n"
    + "Via: MSNSLP/1.0/TLP ;branch=" + aBranchUid + "\r\n"
    + "CSeq: " + aCSeq + "\r\n"
    + "Call-ID: " + aCallId + "\r\n"
    + "Max-Forwards: 0\r\n"
    + "Content-Type: application/x-msnmsgr-" + aContentType + "\r\n"
    + "Content-Length: " + aContent.length + "\r\n"
    + "\r\n"
    + aContent;
  return slpMsg;
}

P2P.prototype.inviteForFileTransfer = function(aContactId, aFile) {
  var sessionId = Math.round(Math.random() * 100000);
  this.sessions[sessionId] = new P2PSession(sessionId, aContactId, this);
  var session = this.sessions[sessionId];
  session.inviteForFileTransfer(aFile);
}

P2P.prototype.parseBinaryHeaders = function(aHeaders) {
  var headers = {};
  headers.sessionId = convert4LeBytesToInt(aHeaders.substr(0, 4));
  headers.id = convert4LeBytesToInt(aHeaders.substr(4, 4));
  headers.dataOffset = convert4LeBytesToInt(aHeaders.substr(8, 4));
  headers.msgLength = convert4LeBytesToInt(aHeaders.substr(16, 4));
  headers.dataSize = convert4LeBytesToInt(aHeaders.substr(24, 4));
  headers.flags = convert4LeBytesToInt(aHeaders.substr(28, 4));
  headers.dWord1 = convert4LeBytesToInt(aHeaders.substr(32, 4));
  headers.dWord2 = convert4LeBytesToInt(aHeaders.substr(36, 4));
  headers.qWord = convert4LeBytesToInt(aHeaders.substr(40, 8));
  return headers;
}

P2P.prototype.parseSlpMessage = function(aMsg) {
  var msg = {};
  var startLineEndPos = aMsg.indexOf("\r\n");
  msg.startLine = aMsg.substr(0, startLineEndPos);
  var payloadLength = aMsg.length - startLineEndPos - 2;
  var payload = aMsg.substr(startLineEndPos + 1, payloadLength);
  var payloadParts = payload.split("\r\n\r\n");
  msg.headers = Utils.parseHeaders(payloadParts[0]);
  msg.content = Utils.parseHeaders(payloadParts[1]);
  return msg;
}

P2P.prototype.requestDisplayPicture = function(aContact) {
  var sessionId = Math.round(Math.random() * 100000);
  this.sessions[sessionId] = new P2PSession(sessionId, aContact.email, this);
  var session = this.sessions[sessionId];
  session.requestDisplayPicture(aContact.msnObject.str);
}

P2P.prototype.send200Message = function(aContactId, aBranchUid, aCallId, aSessionId) {
  status = "200 OK";
  this.sendSlpStatusMessage(status, aContactId, aBranchUid, aCallId, aSessionId);
}

P2P.prototype.send603Message = function(aContactId, aBranchUid, aCallId, aSessionId) {
  status = "603 Decline";
  this.sendSlpStatusMessage(status, aContactId, aBranchUid, aCallId, aSessionId);
}

P2P.prototype.sendAcknowledgement = function(aContactId, aAckBinaryHeader) {
  var binaryHeader = aAckBinaryHeader.substr(0, 4)
    + convertIntTo4LeBytes(++this.baseId)
    + "\0\0\0\0\0\0\0\0"
    + aAckBinaryHeader.substr(16, 8)
    + "\0\0\0\0"
    + convertIntTo4LeBytes(2)
    + aAckBinaryHeader.substr(4, 4)
    + aAckBinaryHeader.substr(32, 4)
    + aAckBinaryHeader.substr(16, 8);
  var p2pMsg = binaryHeader + BINARY_FOOTER_0;
  dump("Sending acknowledgement...\n");
  this.sbSession.sendDMessage(aContactId, p2pMsg);
}

P2P.prototype.sendByeRequest = function(aContactId, aCallId) {
  var slpMsgContent = "\r\n\0";
  var startLine = "BYE MSNMSGR:" + aContactId + " MSNSLP/1.0";
  var branchUid = Utils.getUniversallyUniqueId();
  var cSeq = 0;
  var contentType = "sessionclosebody";
  var slpMsg = this.getSlpMessage(startLine, aContactId, branchUid, cSeq, aCallId, contentType, slpMsgContent);
  var binaryHeader = this.getBinaryHeaders(0, ++this.baseId, 0, slpMsg.length, slpMsg.length, 0);
  var p2pMsg = binaryHeader + slpMsg + BINARY_FOOTER_0;
  this.sbSession.sendDMessage(aContactId, p2pMsg);
}

P2P.prototype.sendDataMessage = function(aSessionId, aContactId, aDataOffset, aMsgLength, aDataSize, aAppId, aData) {
  var flags = 0;
  if (aAppId == DISPLAY_PICTURE_APP_ID_1 || aAppId == DISPLAY_PICTURE_APP_ID_2) {
    flags = FLAG_NOTIFY_UI;
  }
  else if (aAppId == FILE_TRANSFER_APP_ID) {
    flags = FLAG_SAVE_TO_FILE + FLAG_NOTIFY_UI;
  }
  var id = (aDataOffset == 0) ? ++this.baseId : this.baseId;
  var binaryHeader = this.getBinaryHeaders(aSessionId, id, aDataOffset, aMsgLength, aDataSize, flags);
  var binaryFooter = "\0\0\0" + String.fromCharCode(aAppId);
  var p2pMsg = binaryHeader + aData + binaryFooter;
  dump("Sending data message...\n");
  this.sbSession.sendDMessage(aContactId, p2pMsg);
}

P2P.prototype.sendDataPreparationMessage = function(aSessionId, aContactId, aAppId) {
  var flags = 0;
  if (aAppId == DISPLAY_PICTURE_APP_ID_1 || aAppId == DISPLAY_PICTURE_APP_ID_2) {
    flags = FLAG_NOTIFY_UI;
  }
  else if (aAppId == FILE_TRANSFER_APP_ID) {
    flags = FLAG_SAVE_TO_FILE + FLAG_NOTIFY_UI;
  }
  var binaryHeader = this.getBinaryHeaders(aSessionId, ++this.baseId, 0, 4, 4, flags);
  var binaryFooter = "\0\0\0" + String.fromCharCode(aAppId);
  var p2pMsg = binaryHeader + "\0\0\0\0" + binaryFooter;
  dump("Sending data preparation message...\n");
  this.sbSession.sendDMessage(aContactId, p2pMsg);
}

P2P.prototype.sendInviteRequest = function(aContactId, aCallId, aEufGuid, aSessionId, aAppId, aContext) {
  var startLine = "INVITE MSNMSGR:" + aContactId + " MSNSLP/1.0";
  var context64 = btoa(aContext);
  var content = "EUF-GUID: " + aEufGuid + "\r\n"
    + "SessionID: " + aSessionId + "\r\n"
    + "AppID: " + aAppId + "\r\n"
    + "Context: " + context64 + "\r\n"
    + "\r\n\0";
  var branchUid = Utils.getUniversallyUniqueId();
  var cSeq = 0;
  var contentType = "sessionreqbody"
  var slpMsg = this.getSlpMessage(startLine, aContactId, branchUid, cSeq, aCallId, contentType, content);
  var id = this.baseId;
  var binaryHeader = this.getBinaryHeaders(0, id, 0, slpMsg.length, slpMsg.length, 0);
  var p2pMsg = binaryHeader + slpMsg + BINARY_FOOTER_0;
  this.toAck[id] = aSessionId;
  this.sbSession.sendDMessage(aContactId, p2pMsg);
}

P2P.prototype.sendSlpStatusMessage = function(aStatus, aContactId, aBranchUid, aCallId, aSessionId) {
  var slpMsgContent = "SessionID: " + aSessionId + "\r\n"
    + "\r\n\0";
  var startLine = "MSNSLP/1.0 " + aStatus;
  var cSeq = 1;
  var contentType = "sessionreqbody";
  var slpMsg = this.getSlpMessage(startLine, aContactId, aBranchUid, cSeq, aCallId, contentType, slpMsgContent);
  var id = ++this.baseId;
  var binaryHeader = this.getBinaryHeaders(0, id, 0, slpMsg.length, slpMsg.length, 0);
  var p2pMsg = binaryHeader + slpMsg + BINARY_FOOTER_0;
  this.toAck[id] = aSessionId;
  this.sbSession.sendDMessage(aContactId, p2pMsg);
}

P2P.prototype.showFileTransferStatus = function(aStatus) {
  this.sbSession.showFileTransferStatus(aStatus);
}

P2P.prototype.showFileTransferProgress = function(aPercentage) {
  this.sbSession.showFileTransferProgress(aPercentage);
}

function convertIntTo4LeBytes(aInteger) {
  var bytes = String.fromCharCode(aInteger & 255, (aInteger >> 8) & 255, (aInteger >> 16) & 255, (aInteger >> 24) & 255);
  return bytes;
}

function convert4LeBytesToInt(aBytes) {
  var integer = (aBytes.charCodeAt(0) | (aBytes.charCodeAt(1) << 8) | (aBytes.charCodeAt(2) << 16) | (aBytes.charCodeAt(3) << 24));
  return integer;
}