var Emoticons = {};
Emoticons[":@"] = "angry.png";
Emoticons[":-@"] = "angry.png";
Emoticons[":D"] = "bigsmile.png";
Emoticons[":-D"] = "bigsmile.png";
Emoticons[":S"] = "confused.png";
Emoticons[":-S"] = "confused.png";
Emoticons[":'("] = "crying.png";
Emoticons[":|"] = "disappointed.png";
Emoticons[":-|"] = "disappointed.png";
Emoticons[":)"] = "happy.png";
Emoticons[":-)"] = "happy.png";
Emoticons[":("] = "sad.png";
Emoticons[":-("] = "sad.png";
Emoticons[":P"] = "stickouttongue.png";
Emoticons[":-P"] = "stickouttongue.png";
Emoticons[":O"] = "surprised.png";
Emoticons[":-O"] = "surprised.png";
Emoticons[";)"] = "wink.png";
Emoticons[";-)"] = "wink.png";

function onSessionLoad() {
  onEditFontSizeCommand("small");
  document.getElementById("message").focus();
}

function onSessionUnload() {
  window.session.onSessionWindowUnloaded();
}

function onFileTransferCancelCommand() {
  window.session.cancelFileTransfer();
}

function onEditFontSizeCommand(aClass) {
  var chatPanel = document.getElementById("chat-panel");
  chatPanel.contentDocument.body.setAttribute("class", aClass);
  document.getElementById("message").setAttribute("class", aClass);
}

function onContextMenu() {
  var element = document.popupNode;
  var isLink = element instanceof HTMLAnchorElement;
  document.getElementById("cmd-copy").hidden = isLink;
  document.getElementById("cmd-copyLink").hidden = !isLink;
}

function onInviteContactCommand() {
  window.session.inviteContact();
}

function onLinkClicked(aEvent) {
  window.session.openLink(this.href);
  aEvent.preventDefault();
}

function onLogCommand() {
  var logMenuItem = document.getElementById("window-log");
  var logging = logMenuItem.getAttribute("checked");
  if (logging) {
    window.session.turnLoggingOn();
  }
  else {
    window.session.turnLoggingOff();
  }
}

function onMessageBoxKeyPress(aEvent) {
  if (aEvent.type == "keypress" && aEvent.keyCode == aEvent.DOM_VK_RETURN
      && !aEvent.shiftKey) {
    var messageBox = document.getElementById("message");
    var msgText = messageBox.value;
    if (msgText) {
      messageBox.value = "";
      window.session.sendIM(msgText);
    }
    aEvent.preventDefault();
  }
  else {
    window.session.onTyping();
  }
}

function onMessageStyleChanged(aStyle) {
  document.getElementById("message").setAttribute("style", aStyle);
}

function onSessionClosed() {
  document.getElementById("message").disabled = true;
}

function onSendFileCommand() {
  window.session.sendFile();
}

function scrollPanel() {
  var chatPanel = document.getElementById("chat-panel");
  var chatPanelView = chatPanel.contentDocument.defaultView;
  if (chatPanelView.scrollY != chatPanelView.scrollMaxY) {
    chatPanelView.scroll(0, chatPanelView.scrollMaxY);
  }
}

function setDisplayPicture(aFilePath) {
  document.getElementById("display-picture").src = aFilePath;
}

function setFileTransferProgress(aPercentage) {
  document.getElementById("file-transfer-progress")
          .setAttribute("value", aPercentage);
}

function setFileTransferStatus(aStatusText) {
  document.getElementById("file-transfer-status").value = aStatusText;
}

function setHeaderColor(aHeaderColor) {
  var chatParticipantsBar = document.getElementById("chat-participants");
  chatParticipantsBar.style.backgroundColor = aHeaderColor;
}

function setHeaderText(aHeaderText) {
  var chatParticipantsBar = document.getElementById("chat-participants");
  chatParticipantsBar.firstChild.nodeValue = aHeaderText;
}

function setSession(session) {
  window.session = session;
}

function setStatusMessage(aStatusMsg) {
  var statusBar = document.getElementById("status-bar-main").firstChild;
  statusBar.setAttribute("label", aStatusMsg);
}

function setTitleText(aTitleText) {
  document.title = aTitleText;
}

function showFileTransferBox(aShow) {
  document.getElementById("file-transfer-box").collapsed = !aShow;
  scrollPanel();
}

function showIM(aContactId, aDisplayName, aMsgText, aStyle, aMyMsg) {
  var msgNode = createMessageNode(aDisplayName, aMsgText, aStyle, aMyMsg);
  appendAndScroll(msgNode);
}

function showNotificationMessage(aMsg) {
  var msgNode = document.createElementNS(XHTML, "div");
  msgNode.setAttribute("class", "notification-message");
  var textNode = document.createTextNode(aMsg);
  msgNode.appendChild(textNode);
  appendAndScroll(msgNode);
}

function appendAndScroll(aNode) {
  var panel = document.getElementById("chat-panel").contentDocument;
  var panelView = panel.defaultView;
  var scroll = (panelView.scrollY == panelView.scrollMaxY);
  panel.body.appendChild(aNode);
  if (scroll) {
    panelView.scroll(0, panelView.scrollMaxY);
  }
}

function createMessageNode(aFrom, aMsgText, aStyle, aMyMsg) {
  var msgNode = document.createElementNS(XHTML, "div");
  var fromNode = document.createElementNS(XHTML, "div");
  var fromTextNode = document.createTextNode(aFrom + ":");
  var msgTextNode = createMessageTextNode(aMsgText);
  fromNode.appendChild(fromTextNode);
  if (aMyMsg) {
    fromNode.setAttribute("class", "from fromme");        
  }
  else {
    fromNode.setAttribute("class", "from");
  }
  msgTextNode.setAttribute("class", "message");
  msgTextNode.setAttribute("style", aStyle);
  msgNode.appendChild(fromNode);
  msgNode.appendChild(msgTextNode);
  msgNode.setAttribute("class", "instant-message");
  return msgNode;
}

function createMessageTextNode(aMsgText) {
  var msgTextNode = document.createElementNS(XHTML, "div");
  var lines = aMsgText.split("\n");
  for (i in lines) {
    var words = lines[i].split(" ");
    for (j in words) {
      msgTextNode.appendChild(mungeText(words[j]));
      if (j < words.length) {
        msgTextNode.appendChild(document.createTextNode(" "));
      }
    }
    if (i < lines.length) {
      msgTextNode.appendChild(document.createTextNode("\n"));
    }
  }

  function mungeText(aText) {
    var node = null;
    if (aText.toUpperCase() in Emoticons) {
      node = document.createElementNS(XHTML, "img");
      node.setAttribute("src", "chrome://msnmsgr/skin/emoticons/"
                        + Emoticons[aText.toUpperCase()]);
      node.setAttribute("alt", aText);
      node.setAttribute("width", 19);
      node.setAttribute("height", 19);
    }
    else {
      var linkRegExp = /^https?:\/\/|^www\d?|^ftp:\/\//;
      if (linkRegExp.exec(aText)) {
        var textNode = document.createTextNode(aText);
        node = document.createElementNS(XHTML, "a");
        node.setAttribute("href", aText);
        node.appendChild(textNode);
        node.addEventListener("click", onLinkClicked, true);
      }
      else {
        node = document.createTextNode(aText);
      }
    }
    return node;
  }

  return msgTextNode;
}